@@ -35,6 +35,10 @@
 #endif
 #include "mhpdbg.h"
 
+#include "mapping.h"
+extern void mapping_init(void);
+extern void mapping_close(void);
+
 
 /*
  * XXX - the mem size of 734 is much more dangerous than 704. 704 is the
@@ -350,6 +354,7 @@ void dump_config_status(void *printfunc)
     }
     else (*print)("dumping the current runtime configuration:\n");
     (*print)("Version: dosemu-" VERSTR " versioncode = 0x%08x\n\n", DOSEMU_VERSION_CODE);
+    (*print)("Running Kernel Version: linux-%d.%d.%d\n", kernel_version_code >> 16, (kernel_version_code >> 8) & 255, kernel_version_code & 255);
     (*print)("cpu ");
     switch (vm86s.cpu_type) {
       case CPU_386: s = "386"; break;
@@ -375,7 +380,12 @@ void dump_config_status(void *printfunc)
     (*print)("cpuemu %d\n", config.cpuemu);
 #endif
 
-    (*print)("mappingdriver %s\n", config.mappingdriver ? config.mappingdriver : "auto");
+    if (config_check_only) mapping_init();
+    if (mappingdriver.name)
+      (*print)("mappingdriver %s\n", mappingdriver.name);
+    else
+      (*print)("mappingdriver %s\n", config.mappingdriver ? config.mappingdriver : "auto");
+    if (config_check_only) mapping_close();
     (*print)("hdiskboot %d\nmem_size %d\n",
         config.hdiskboot, config.mem_size);
     (*print)("ems_size 0x%x\nems_frame 0x%x\nsecure %d\n",
@@ -431,6 +441,9 @@ void dump_config_status(void *printfunc)
       case MATROX: s = "matrox"; break;
       case WDVGA: s = "wdvga"; break;
       case SIS: s = "sis"; break;
+#ifdef USE_SVGALIB
+      case SVGALIB: s = "svgalib"; break;
+#endif
       default: s = "unknown"; break;
     }
     (*print)("config.X %d\nhogthreshold %d\nchipset \"%s\"\n",
@@ -1054,7 +1067,7 @@ int parse_debugflags(const char *s, unsigned char flag)
     const char      allopts[] = "dARWDCvkiTtsm#pQgcwhIExMnPrSeZ";
 #endif
 /*    abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ  */
-/*      *** *** * ** * **  ***  * ***   *   *  *****  ** *  */
+/*      *** *** * ** * *** ***  * ***   *   *  *****  ** *  */
 
     /*
      * if you add new classes of debug messages, make sure to add the
@@ -1172,6 +1185,8 @@ int parse_debugflags(const char *s, unsigned char flag)
 	case 'e':		/* cpu-emu */
 	    d.emu = flag;
 	    break;
+#endif
+#ifdef TRACE_DPMI
 	case 't':		/* dpmi */
 	    d.dpmit = flag;
 	    break;
@@ -1182,14 +1197,15 @@ int parse_debugflags(const char *s, unsigned char flag)
 		newopts[0] = flag ? '+' : '-';
 		newopts[1] = 0;
 		strcat(newopts, allopts);
-#if 1 /* we need to _allways_ remove that flag here !!! #ifdef X86_EMULATOR */
-		/* hack-do not set 'e' flag if not explicitly specified */
-		{char *p=newopts; while (*p) {if (*p=='e') *p='g'; p++;}}
+#if 1 /* we need to _always_ remove these flags here !!! #ifdef X86_EMULATOR */
+		/* hack-do not set 'e,t' flags if not explicitly specified */
+		{char *p=newopts;
+		 while (*p) {if ((*p=='e')||(*p=='t')) *p='g'; p++;}}
 #endif
 		parse_debugflags(newopts, flag);
 		free(newopts);
-		break;
 	    }
+	    break;
 	case '0'...'9':	/* set debug level, 0 is off, 9 is most
 				 * verbose */
 	    flag = c - '0';
