@@ -208,7 +208,7 @@ void hardware_setup(void)
   pic_unmaski(PIC_IRQ1);
   pic_seti(PIC_IRQ8, rtc_int8, 0);
   pic_unmaski(PIC_IRQ8);
-  if (mice->intdrv || mice->type == MOUSE_PS2) {
+  if (mice->intdrv || mice->type == MOUSE_PS2 || mice->type == MOUSE_IMPS2) {
     pic_seti(PIC_IMOUSE, DOSEMUMouseEvents, 0);
     pic_unmaski(PIC_IMOUSE);
   }
@@ -349,7 +349,7 @@ static inline void bios_mem_setup(void)
   CONF_NFLOP(configuration, config.fdisks);
   CONF_NSER(configuration, config.num_ser);
   CONF_NLPT(configuration, config.num_lpt);
-  if ((mice->type == MOUSE_PS2) || (mice->intdrv))
+  if ((mice->type == MOUSE_PS2) || (mice->type == MOUSE_IMPS2) || (mice->intdrv))
     configuration |= CONF_MOUSE;
 
   if (config.mathco)
@@ -467,7 +467,7 @@ void device_init(void)
 		     console_speaker_on, console_speaker_off);
   }
   iodev_init();
-  mouse_init();
+  dosemu_mouse_init();
   printer_init();
   disk_init();
 }
