@@ -154,7 +154,6 @@ void X_keycode_process_key(XKeyEvent *e)
 	static const int * X_scan = 0;
 	static int direct_scan=0, scan_end=0;
 	unsigned int scan;
-	int i;
 
 	if(!scan_test) {
 		if(XKeysymToKeycode(display, XK_Down) == 93) { /* New keyboard */
@@ -184,16 +183,20 @@ void X_keycode_process_key(XKeyEvent *e)
 			scan=0;
 	X_printf("X_keycode_process_key: keycode=%d, scancode=%d, %s\n",
 		 e->keycode,scan,(e->type==KeyRelease)?"released":"pressed");
+#if 0	/* this seems to be wrong, caps/num lock doesn't works with it
+	 * 			-- Herbert Xu, 2000/07/22
+	 */
 	if (scan == 0x3a || scan == 0x45) {
 		/*
 		 * bring our lock flags in sync with the state of the server
 		 */
-		i = shiftstate & ((scan == 0x3a) ? CAPS_LOCK : NUM_LOCK);
+		int i = shiftstate & ((scan == 0x3a) ? CAPS_LOCK : NUM_LOCK);
 		if ((e->type==KeyPress && !i) || (e->type==KeyRelease && i)) {
 			put_keycode(scan, 0);
 			put_keycode(scan, 1);
 		}
 	} else
+#endif
 		put_keycode(scan, (e->type==KeyRelease));
 	return;
 }
