@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -19,15 +19,29 @@
 #include <string.h>
 #include <sys/ioctl.h>
 #ifdef __linux__
-  #include "Linux/hdreg.h"
-  #include "Linux/fd.h"
-  #include "Linux/fs.h"
+  #include <linux/hdreg.h>
+  #include <linux/fd.h>
+  #include "kversion.h"
+  #if (LX_KERNEL_VERSION < 2003099)
+    #include <linux/fs.h>
+  #else
+    #if (GLIBC_VERSION_CODE >= 2000)
+      #include <sys/mount.h>
+    #else
+      #ifndef BLKGETSIZE
+			/* Note: _IO() being defined in <sys/ioctl.h> */
+        #define BLKGETSIZE _IO(0x12,96) /* return device size */
+      #endif
+    #endif
+  #endif
 #endif
 #include <sys/stat.h>
 #include <sys/time.h>
 
 #include "emu.h"
 #include "disks.h"
+#include "timers.h"
+#include "utilities.h"
 #include "priv.h"
 #include "dosemu_select.h"
 #include "int.h"
@@ -115,7 +129,7 @@ static void dump_disk_blks(unsigned char *tb, int count, int ssiz)
  *
  *   |   adds up to 1 sector (512 bytes) of Master Boot Record (MBR)
  *       THIS IS KEPT IN MEMORY, and is the file 
- *       <DOSEMU_LIB_DIR>/partition.<partition>.
+ *       /var/lib/dosemu/partition.<partition>.
  *
  *   .   is any number of sectors before the desired partition start address.
  *       THESE SECTORS ARE NOT EMULATED, BUT MERELY SKIPPED
