@@ -1,12 +1,9 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
 
-#ifdef __NetBSD__
-#define EDEADLOCK EDEADLK
-#endif
 
 #include <features.h>
 #include <stdio.h>
@@ -28,11 +25,6 @@
 #include <limits.h>
 #include <assert.h>
 
-#ifdef __NetBSD__
-#include <signal.h>
-#include <machine/pcvt_ioctl.h>
-#include "netbsd_vm86.h"
-#endif
 #ifdef __linux__
 #include <signal.h>
 #if GLIBC_VERSION_CODE >= 2000
@@ -48,19 +40,6 @@
 
 #include <sys/socket.h>
 #include <sys/ioctl.h>
-#ifndef __NetBSD__
-#include <linux/if_ether.h>
-#endif
-#ifdef __NetBSD__
-#include <sys/mman.h>
-#else
-extern caddr_t mmap __P ((caddr_t __addr, size_t __len,
-                          int __prot, int __flags, int __fd, off_t __off));
-                          extern int munmap __P ((caddr_t __addr, size_t
-__len));
-#include <linux/mman.h>
-#endif
-
 
 
 #include "config.h"
@@ -88,9 +67,6 @@ __len));
 #include "bitops.h"
 #include "pic.h"
 #include "dpmi.h"
-#ifdef __NetBSD__
-#include <setjmp.h>
-#endif
 
 #include "keyb_clients.h"
 
@@ -112,7 +88,7 @@ inline int process_interrupt(SillyG_t *sg);
 
 
 void io_select_init(void);
-#if defined(SIG) && defined(REQUIRES_VM86PLUS)
+#if defined(SIG)
 int SillyG_pendind_irq_bits=0;
 
 int SillyG_do_irq(void)
@@ -136,7 +112,7 @@ inline int process_interrupt(SillyG_t *sg)
 }
 
 
-#if defined(SIG) && defined(REQUIRES_VM86PLUS)
+#if defined(SIG)
 inline void irq_select()
 {
   if (SillyG) {
@@ -172,7 +148,7 @@ io_select(fd_set fds)
   tvptr.tv_sec=0L;
   tvptr.tv_usec=0L;
 
-#if defined(SIG) && defined(REQUIRES_VM86PLUS)
+#if defined(SIG)
   irq_select();
 #endif
 
@@ -194,7 +170,8 @@ io_select(fd_set fds)
 
     default:			/* has at least 1 descriptor ready */
 
-      if ((mice->intdrv || mice->type == MOUSE_PS2) && mice->fd >= 0)
+      if ((mice->intdrv || mice->type == MOUSE_PS2 || mice->type == MOUSE_IMPS2)
+	  && mice->fd >= 0)
 	if (FD_ISSET(mice->fd, &fds)) {
 		m_printf("MOUSE: We have data\n");
 	  pic_request(PIC_IMOUSE);
