@@ -1691,7 +1691,7 @@ mouse_sethandler(void *f, us * cs, us * ip)
  * DANG_END_FUNCTION
  */
 void
-mouse_init(void)
+dosemu_mouse_init(void)
 {
   PRIV_SAVE_AREA
   serial_t *sptr=NULL;
@@ -1749,7 +1749,8 @@ mouse_init(void)
       DOSEMUSetupMouse();
       memcpy(p,mouse_ver,sizeof(mouse_ver));
     }
-    else if ((mice->type == MOUSE_PS2) || (mice->type == MOUSE_BUSMOUSE)) {
+    else if ((mice->type == MOUSE_PS2) || (mice->type == MOUSE_IMPS2) ||
+	     (mice->type == MOUSE_BUSMOUSE)) {
       enter_priv_on();
       mice->fd = open(mice->dev, O_RDWR | O_NONBLOCK);
       leave_priv_setting();
@@ -1796,7 +1797,7 @@ mouse_init(void)
 }
 
 void
-mouse_close(void)
+dosemu_mouse_close(void)
 {
   int result;
 
@@ -1815,8 +1816,8 @@ mouse_close(void)
     return;
   }
 
-  if (((mice->type == MOUSE_PS2) || (mice->type == MOUSE_BUSMOUSE))
-	&& (mice->fd != -1))
+  if (((mice->type == MOUSE_PS2) || (mice->type == MOUSE_IMPS2) ||
+       (mice->type == MOUSE_BUSMOUSE)) && (mice->fd != -1))
     close(mice->fd);
 }
 
