@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -14,9 +14,6 @@
 #ifdef __linux__
 #include <sys/kd.h>
 #endif
-#ifdef __NetBSD__
-#include <machine/pcvt_ioctl.h>
-#endif
 
 
 void console_speaker_on(void *gp, unsigned ms, unsigned short period)
