@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -68,36 +68,14 @@ check if a name is a special msdos reserved name
 char *is_reserved_msdos(char *fname)
 {
   static char upperFname[13];
-  char *p , *p_;
-
-  /*
-   * LPTx e.t.c. is reserved no matter the path (e.g. .\LPT1 _is_ reserved),
-   * but mfs.c sends an entire path.
-   *             -- Rob Clark <rclark@list-clark.com> 2000/05/09
-   */
-  p = strrchr(fname,'\\');
-  p_ = strrchr(fname,'/');
-  if (p_ > p) p = p_;	/* take what ever is nearer to the end */
-  if (p) fname = p+1;	/* strip the directory part */
+  char *p;
 
   StrnCpy (upperFname, fname, 12);
 
-#if 0
-  /*
-   * The below (now #ifdef'ed out) code strips the extension off for checking.
-   * But the assumption, that file names with a reserved basename remain
-   * reserved even if they have an extension, seems not to be true.
-   * For example CLIPPER-programs _are_ using file names such as
-   * 'LPTx.PRN' for internal spooling purposes.
-   *             -- Bernd Schueler b.schueler@gmx.de 2001/02/23
-   */
-
   /* lpt1.txt and con.txt etc are also illegal */
   p=strchr(upperFname,'.');
   if (p)
    *p='\0';
-#endif
-
   strupperDOS (upperFname);
   if ((strcmp(upperFname,"CLOCK$") == 0) ||
     (strcmp(upperFname,"CON") == 0) ||
