@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -13,7 +13,7 @@ March 1995
 Modified by O.V.Zhirov, July 1998
 */
 
-#if defined(__linux__) || defined(__NetBSD__)
+#if defined(__linux__)
 #define DOSEMU 1		/* this is a port to dosemu */
 #endif
 
