@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -31,11 +31,11 @@
  * The original code was written by Joel N. Weber II. See README.sound
  * for more information. I (Alistair MacDonald) made the code compile, and
  * added a few extra features. I took the code and continued development on
- * 0.61. Michael Beck did a lot of work on 0.60.4, including the original
- * NetBSD code and capability detection under Linux. This code is basically
+ * 0.61. Michael Beck did a lot of work on 0.60.4, including capability
+ * detection under Linux. This code is basically
  * my 0.61 code, brought back into mainstream DOSEmu (0.63), but with
  * Michael's code where I thought it was better than mine. I also separated
- * out the NetBSD & Linux specific driver code. - Alistair
+ * out the Linux specific driver code. - Alistair
  *
  * For 0.66 we have updated many areas and re-organised others to make it 
  * easier to maintain. Many thanks for David for finding so many bugs ....
@@ -60,8 +60,6 @@
  * See the file README.sound in this directory for more information 
  */
 
-/* DANG_FIXME NetBSD SB code will not compile */
-
 /* Uncomment following to force complete emulation of some varient of
  * Creative Technology's SB sound card
  *
@@ -511,7 +509,7 @@ Bit8u sb_get_mixer_IRQ_status (void)
 
 Bit8u adlib_io_read(ioport_t port)
 {
-  Bit8u result;
+  Bit8u result = 0xFF;
 
   /* Adlib Base Port is 0x388 */
   /* Adv. Adlib Base Port is 0x38A */
@@ -540,8 +538,8 @@ Bit8u adlib_io_read(ioport_t port)
 
 Bit8u fm_io_read (ioport_t port)
 {
-  extern struct adlib_info_t adlib_info;
-  /* extern struct adlib_timer_t adlib_timers[2]; - For reference - AM */
+  /* extern struct adlib_info_t adlib_info; - For reference - AM */
+  extern struct adlib_timer_t adlib_timers[2];
   Bit8u retval;
 
   switch (port){
@@ -2203,6 +2201,12 @@ int sb_dma_handler (int status, Bit16u amount)
     S_printf ("SB: In DMA Handler\n");
   }
 
+  if (!SB_info.speaker) {
+    /* No output possbile - Try to stop processing ! */
+    dma_drop_DREQ(config.sb_dma);
+    return DMA_HANDLER_NOT_OK;
+  }
+
   switch (status) {
   case DMA_HANDLER_READ:
     dma_assert_DACK(config.sb_dma);
@@ -2221,7 +2225,7 @@ int sb_dma_handler (int status, Bit16u amount)
        till the next block is started. On auto-init, next block start
        on an EOI. - MK */
         
-    dma_drop_DREQ(config.sb_dma);
+    /*    dma_drop_DREQ(config.sb_dma); */
     if(SB_dsp.bytes_left)
     {
       /* Still some bytes left till IRQ */
@@ -2512,7 +2516,7 @@ static void sb_reset (void)
 
 static void fm_reset (void)
 {
-  extern struct adlib_info_t adlib_info;
+  /* extern struct adlib_info_t adlib_info; - For reference - AM */
   extern struct adlib_timer_t adlib_timers[2];
 
   S_printf ("SB: Resetting FM\n");
