@@ -43,7 +43,7 @@
 #include "pic.h"
 #include "cpu-emu.h"
 #include "emu86.h"
-#include "codegen.h"
+#include "codegen-x86.h"
 #include "dpmi.h"
 
 #define e_set_flags(X,new,mask) \
@@ -68,7 +68,10 @@ static unsigned long sigEMUdelta = 0;
 static int last_emuretrace;
 
 /* This needs to be merged someday with 'mode' */
-int    CEmuStat = 0;
+int CEmuStat = 0;
+
+int JumpOpt = 3;	/* b0=back b1=fwd */
+unsigned long JumpOptLim = 0x1000;
 
 /* Another signal pending flag... this one is used by the compiled
  * back jumps to avoid looping forever */
@@ -81,11 +84,11 @@ int e_sigpa_count;
 
 int emu_dpmi_retcode = -1;
 int in_dpmi_emu = 0;
-int emu_under_X = 0;
 
 SynCPU	TheCPU;
 
 int Running = 0;
+int InCompiledCode = 0;
 
 unsigned long trans_addr, return_addr;	// PC
 
@@ -291,7 +294,10 @@ char *e_print_regs(void)
 	exprintl(rEDI,buf,(ERB_L2+ERB_LEFTM)+13);
 	exprintl(rEBP,buf,(ERB_L2+ERB_LEFTM)+26);
 	exprintl(rESP,buf,(ERB_L2+ERB_LEFTM)+39);
-	exprintl(TheCPU.veflags,buf,(ERB_L3+ERB_LEFTM));
+	if (TheCPU.eflags&EFLAGS_VM)
+		exprintl(TheCPU.veflags,buf,(ERB_L3+ERB_LEFTM));
+	else
+		exprintl(dpmi_eflags,buf,(ERB_L3+ERB_LEFTM));
 	exprintw(TheCPU.cs,buf,(ERB_L3+ERB_LEFTM)+13);
 	exprintw(TheCPU.ds,buf,(ERB_L3+ERB_LEFTM)+26);
 	exprintw(TheCPU.es,buf,(ERB_L3+ERB_LEFTM)+39);
@@ -299,7 +305,7 @@ char *e_print_regs(void)
 	exprintw(TheCPU.gs,buf,(ERB_L4+ERB_LEFTM)+13);
 	exprintw(TheCPU.ss,buf,(ERB_L4+ERB_LEFTM)+26);
 	exprintl(TheCPU.eflags,buf,(ERB_L4+ERB_LEFTM)+39);
-	if (TheCPU.mode & DSPSTK) {
+	if ((d.emu>6) && (TheCPU.mode & DSPSTK)) {
 		int i;
 		unsigned short *stk = (unsigned short *)(LONG_SS+TheCPU.esp);
 		for (i=(ERB_L5+ERB_LEFTM); i<(ERB_L6-2); i+=5) {
@@ -333,7 +339,10 @@ char *e_print_scp_regs(struct sigcontext_struct *scp, int pmode)
 	exprintl(_edi,buf,(ERB_L2+ERB_LEFTM)+13);
 	exprintl(_ebp,buf,(ERB_L2+ERB_LEFTM)+26);
 	exprintl(_esp,buf,(ERB_L2+ERB_LEFTM)+39);
-	exprintl(TheCPU.veflags,buf,(ERB_L3+ERB_LEFTM));
+	if (pmode & 1)
+		exprintl(dpmi_eflags,buf,(ERB_L3+ERB_LEFTM));
+	else
+		exprintl(TheCPU.veflags,buf,(ERB_L3+ERB_LEFTM));
 	exprintw(_cs,buf,(ERB_L3+ERB_LEFTM)+13);
 	exprintw(_ds,buf,(ERB_L3+ERB_LEFTM)+26);
 	exprintw(_es,buf,(ERB_L3+ERB_LEFTM)+39);
@@ -341,16 +350,21 @@ char *e_print_scp_regs(struct sigcontext_struct *scp, int pmode)
 	exprintw(_gs,buf,(ERB_L4+ERB_LEFTM)+13);
 	exprintw(_ss,buf,(ERB_L4+ERB_LEFTM)+26);
 	exprintl(_eflags,buf,(ERB_L4+ERB_LEFTM)+39);
-	if (pmode) {
+	if (pmode & 2) {
+		buf[(ERB_L4+ERB_LEFTM)+47] = 0;
+	}
+	else {
+	    if (pmode & 1) {
 	        if (Segments[_ss>>3].is_32)
 		    stk = (unsigned short *)(GetSegmentBaseAddress(_ss)+_esp);
 	        else
 		    stk = (unsigned short *)(GetSegmentBaseAddress(_ss)+_LWORD(esp));
-	}
-	else
+	    }
+	    else
 		stk = (unsigned short *)((_ss<<4)+_LWORD(esp));
-	for (i=(ERB_L5+ERB_LEFTM); i<(ERB_L6-2); i+=5) {
-	    exprintw(*stk++,buf,i);
+	    for (i=(ERB_L5+ERB_LEFTM); i<(ERB_L6-2); i+=5) {
+		exprintw(*stk++,buf,i);
+	    }
 	}
 	return buf;
 }
@@ -388,15 +402,16 @@ char *e_emu_disasm(unsigned char *org, int is32)
    return buf;
 }
 
+#ifdef TRACE_DPMI
 char *e_scp_disasm(struct sigcontext_struct *scp, int pmode)
 {
    static char insrep = 0;
-   static unsigned char buf[256];
+   static unsigned char buf[1024];
    static unsigned char frmtbuf[256];
    static unsigned long lasta = 0;
    int rc;
    int i;
-   unsigned char *p, *org, *csp2;
+   unsigned char *p, *pb, *org, *org2, *csp2;
    unsigned int seg;
    unsigned int refseg;
    unsigned int ref;
@@ -417,16 +432,22 @@ char *e_scp_disasm(struct sigcontext_struct *scp, int pmode)
    rc = dis_8086((unsigned long)org, org, frmtbuf, (pmode&&IsSegment32(scp->cs)? 3:0),
    	&refseg, &ref, (pmode? (int)csp2 : 0), 1);
 
-   p = buf + sprintf(buf,"%08lx: ",(long)org);
+   pb = buf;
+   org2 = org;
+   while ((*org2&0xfc)==0x64) org2++;	/* skip most prefixes */
+   if ((d.dpmit>3)||(InterOps[*org2]&2))
+	pb += sprintf(pb,"%s",e_print_scp_regs(scp,pmode));
+
+   p = pb + sprintf(pb,"    %08lx: ",(long)org);
    for (i=0; i<rc && i<8; i++) {
            p += sprintf(p, "%02x", *(org+i));
    }
    sprintf(p,"%20s", " ");
-   sprintf(buf+28, "%04x:%04lx %s\n", scp->cs, scp->eip, frmtbuf);
+   sprintf(pb+28, "%04x:%04lx %s\n", scp->cs, scp->eip, frmtbuf);
 
    return buf;
 }
-
+#endif
 
 #undef FP_DISPHEX
 
@@ -475,47 +496,71 @@ char *e_trace_fp(void)
 static void Reg2Cpu (struct vm86plus_struct *info, int mode)
 {
   char big;	// dummy
-  TheCPU.eax     = info->regs.eax;
-  TheCPU.ebx     = info->regs.ebx;
-  TheCPU.ecx     = info->regs.ecx;
-  TheCPU.edx     = info->regs.edx;
-  TheCPU.esi     = info->regs.esi;
-  TheCPU.edi     = info->regs.edi;
-  TheCPU.ebp     = info->regs.ebp;
-  TheCPU.esp     = info->regs.esp;
-  TheCPU.cs      = info->regs.cs;
-  TheCPU.ds      = info->regs.ds;
-  TheCPU.es      = info->regs.es;
-  TheCPU.ss      = info->regs.ss;
-  TheCPU.fs      = info->regs.fs;
-  TheCPU.gs      = info->regs.gs;
-  TheCPU.err     = 0;
-  TheCPU.eip     = info->regs.eip&0xffff;
+  unsigned long flg;
+ /*
+  * Copy the dosemu flags (in vm86s) into our veflags, which are the
+  * equivalent of the VEFLAGS in /linux/arch/i386/kernel/vm86.c
+  * We'll move back our flags into vm86s when exiting this function.
+  */
+  TheCPU.veflags = info->regs.eflags;
+  info->regs.eflags &= SAFE_MASK;
+  /* get the protected mode flags. Note that RF and VM are cleared
+   * by pushfd (but not by ints and traps) */
+  __asm__ __volatile__ ("
+	pushfl
+	popl	%0"
+	: "=m"(flg) : : "memory");
+  info->regs.eflags |= (flg & notSAFE_MASK);
+  info->regs.eflags |= EFLAGS_VM;
 
   if (config.cpuemu>2) {
     /* a vm86 call switch has been detected.
        Setup flags for the 1st time. */
-    TheCPU.eflags = (TheCPU.veflags&0x2346d5) | 0x20002;
     config.cpuemu=2;
   }
-  else
-    TheCPU.eflags = (info->regs.eflags&0x274ed5) | 0x20002 |
-	(TheCPU.veflags&EFLAGS_VIF? EFLAGS_IF:0);
+
+  if (d.emu>1) e_printf("Reg2Cpu> vm86=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+	info->regs.eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
+  TheCPU.eax     = info->regs.eax;	/* 2c -> 18 */
+  TheCPU.ebx     = info->regs.ebx;	/* 20 -> 00 */
+  TheCPU.ecx     = info->regs.ecx;	/* 28 -> 04 */
+  TheCPU.edx     = info->regs.edx;	/* 24 -> 08 */
+  TheCPU.esi     = info->regs.esi;	/* 14 -> 0c */
+  TheCPU.edi     = info->regs.edi;	/* 10 -> 10 */
+  TheCPU.ebp     = info->regs.ebp;	/* 18 -> 14 */
+  TheCPU.esp     = info->regs.esp;	/* 1c -> 3c */
+  TheCPU.cs      = info->regs.cs;	/* 3c -> 34 */
+  TheCPU.ds      = info->regs.ds;	/* 0c -> 48 */
+  TheCPU.es      = info->regs.es;	/* 08 -> 44 */
+  TheCPU.ss      = info->regs.ss;	/* 48 -> 40 */
+  TheCPU.fs      = info->regs.fs;	/* 04 -> 4c */
+  TheCPU.gs      = info->regs.gs;	/* 00 -> 50 */
+  TheCPU.err     = 0;
+  TheCPU.eip     = info->regs.eip&0xffff;
+  TheCPU.eflags  = info->regs.eflags;	/* 40 -> 38 */
+
   (void)SET_SEGREG(CS_DTR,big,Ofs_CS,TheCPU.cs);
   (void)SET_SEGREG(DS_DTR,big,Ofs_DS,TheCPU.ds);
-  (void)SET_SEGREG(SS_DTR,big,Ofs_SS,TheCPU.ss);
   (void)SET_SEGREG(ES_DTR,big,Ofs_ES,TheCPU.es);
+  (void)SET_SEGREG(SS_DTR,big,Ofs_SS,TheCPU.ss);
   (void)SET_SEGREG(FS_DTR,big,Ofs_FS,TheCPU.fs);
   (void)SET_SEGREG(GS_DTR,big,Ofs_GS,TheCPU.gs);
-  trans_addr   = LONG_CS + TheCPU.eip;
+  trans_addr     = LONG_CS + TheCPU.eip;
 
-  if (d.emu>1) e_printf("Reg2Cpu: %08lx -> %08lx\n%s\n",
-  	info->regs.eflags, TheCPU.eflags, e_print_regs());
+  if (d.emu>1) {
+	if (d.emu==3) e_printf("Reg2Cpu< vm86=%08lx dpm=%08x emu=%08lx evf=%08lx\n%s\n",
+		info->regs.eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags,
+		e_print_regs());
+	else e_printf("Reg2Cpu< vm86=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+		info->regs.eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
+  }
 }
 
 
 static void Cpu2Reg (struct vm86plus_struct *info)
 {
+  if (d.emu>1) e_printf("Cpu2Reg> vm86=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+	info->regs.eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
   info->regs.eax = TheCPU.eax;
   info->regs.ebx = TheCPU.ebx;
   info->regs.ecx = TheCPU.ecx;
@@ -540,33 +585,35 @@ static void Cpu2Reg (struct vm86plus_struct *info)
   info->regs.eflags = (info->regs.eflags & (VIP|VIF)) |
   		      (TheCPU.eflags & (eTSSMASK|0x20ed5)) | 0x10002;
 
-  if (d.emu>1) e_printf("Cpu2Reg: %08lx -> %08lx\n",
-  	TheCPU.eflags, info->regs.eflags);
+  if (d.emu>1) e_printf("Cpu2Reg< vm86=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+	info->regs.eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
 }
 
 
 static void Scp2CpuR (struct sigcontext_struct *scp)
 {
-  memcpy(&TheCPU.FIELD0,scp,sizeof(struct sigcontext_struct));
+  if (d.emu>1) e_printf("Scp2CpuR> scp=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+	scp->eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
+  __memcpy(&TheCPU.FIELD0,scp,sizeof(struct sigcontext_struct));
   TheCPU.err = 0;
 
-  if (in_dpmi) {
-    TheCPU.eflags = (scp->eflags&(eTSSMASK|0x210cd5)) | 0x202;
-    TheCPU.veflags = (TheCPU.veflags&~(VIP|VIF|IF)) | (dpmi_eflags&(VIP|VIF|IF));
+  if (in_dpmi) {	// vm86 during dpmi active
+	e_printf("** Scp2Cpu VM in_dpmi\n");
   }
-  else
-    TheCPU.eflags = (scp->eflags&(eTSSMASK|0x210ed5)) | 0x20002;
+  TheCPU.eflags = (scp->eflags&(eTSSMASK|0x10ed5)) | 0x20002;
   trans_addr = ((scp->cs<<4) + scp->eip);
 
-  if (d.emu>1)
-	e_printf("Scp2CpuR: %08lx -> %08lx\n", scp->eflags, TheCPU.eflags);
+  if (d.emu>1) e_printf("Scp2CpuR< scp=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+	scp->eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
 }
 
 
 static void Cpu2Scp (struct sigcontext_struct *scp,
 	int trapno)
 {
-  memcpy(scp,&TheCPU.FIELD0,sizeof(struct sigcontext_struct));
+  if (d.emu>1) e_printf("Cpu2Scp> scp=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+	scp->eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
+  __memcpy(scp,&TheCPU.FIELD0,sizeof(struct sigcontext_struct));
   scp->trapno = trapno;
   /* Error code format:
    * b31-b16 = 0 (undef)
@@ -578,20 +625,16 @@ static void Cpu2Scp (struct sigcontext_struct *scp,
     scp->cs = TheCPU.cs;
     scp->eip = return_addr - LONG_CS;
     scp->eflags = (dpmi_eflags & (VIP|VIF|IF)) |
-  		      (TheCPU.eflags & (eTSSMASK|0xcd5)) | 0x10002;
-    if (d.emu>1)
-	e_printf("DPM86: flags: %08lx %08x -> %08lx\n",
-		TheCPU.eflags, dpmi_eflags, scp->eflags);
+  		      (TheCPU.eflags & (eTSSMASK|0xed5)) | 0x10002;
   }
   else {
     scp->cs  = return_addr >> 16;
     scp->eip = return_addr & 0xffff;
     scp->eflags = (vm86s.regs.eflags & (VIP|VIF)) |
   		      (TheCPU.eflags & (eTSSMASK|0xed5)) | 0x20002;
-    if (d.emu>1)
-	e_printf("Cpu2Scp: %08lx %08lx -> %08lx\n",
-		TheCPU.eflags, vm86s.regs.eflags, scp->eflags);
   }
+  if (d.emu>1) e_printf("Cpu2Scp< scp=%08lx vm86=%08lx dpm=%08x emu=%08lx evf=%08lx\n",
+	scp->eflags,vm86s.regs.eflags,dpmi_eflags,TheCPU.eflags,TheCPU.veflags);
 }
 
 
@@ -604,16 +647,17 @@ static void Cpu2Scp (struct sigcontext_struct *scp,
 static int Scp2CpuD (struct sigcontext_struct *scp)
 {
   char big; int mode=0;
-  memcpy(&TheCPU.FIELD0,scp,sizeof(struct sigcontext_struct));
+  __memcpy(&TheCPU.FIELD0,scp,sizeof(struct sigcontext_struct));
   TheCPU.veflags = scp->eflags&~EFLAGS_VM;
 
   mode |= ADDR16;	/* just a default */
   TheCPU.err = SET_SEGREG(CS_DTR,big,Ofs_CS,TheCPU.cs);
-  if (big) mode=0; else mode |= (DATA16|STACK16);
+  if (big) mode=0; else mode |= DATA16;
   if (!TheCPU.err) {
     TheCPU.err = SET_SEGREG(DS_DTR,big,Ofs_DS,TheCPU.ds);
     if (!TheCPU.err) {
-      SET_SEGREG(SS_DTR,big,Ofs_SS,TheCPU.ss); if (big) mode&=~STACK16;
+      SET_SEGREG(SS_DTR,big,Ofs_SS,TheCPU.ss);
+      TheCPU.StackMask = (big? 0xffffffff : 0x0000ffff);
       if (!TheCPU.err) {
 	SET_SEGREG(ES_DTR,big,Ofs_ES,TheCPU.es);
 	SET_SEGREG(FS_DTR,big,Ofs_FS,TheCPU.fs);
@@ -621,67 +665,15 @@ static int Scp2CpuD (struct sigcontext_struct *scp)
       }
     }
   }
-  TheCPU.eflags = (scp->eflags&(eTSSMASK|0x210cd5)) | 0x202;
-  TheCPU.veflags = (TheCPU.veflags&~(VIP|VIF|IF)) | (dpmi_eflags&IF) |
-	(scp->eflags&(VIP|VIF));
+  TheCPU.eflags = (scp->eflags&(eTSSMASK|0x10ed5)) | 2;
   trans_addr = LONG_CS + scp->eip;
-  if (d.emu>1)
-	e_printf("Scp2CpuD: %08lx -> %08lx\n\tIP=%08lx:%08lx\n%s\n",
-		scp->eflags, TheCPU.eflags, LONG_CS, scp->eip,
-		e_print_regs());
-  return mode;
-}
-
-
-/* ======================================================================= */
-
-#define SKIP_BOUND	// info only, for the moment
-
-
-int e_decode_bound_excp (unsigned char *csp, struct sigcontext_struct *scp)
-{
-#ifdef USE_BOUND
-  unsigned char modrm,sib;
-  long l, r, *bp = NULL;
-
-  modrm = csp[1];
-  dbug_printf("BOUND exception at %#04x:%#08lx: %02x %02x%02x%02x%02x\n",
-	_cs,_eip, modrm, csp[2], csp[3], csp[4], csp[5]);
-  if (*csp != BOUND) return 2;	// goto blank screen & reset
-
-  /* bound does not occur in 16-bit code */
-  r = ((long *)&(scp->edi))[7-D_MO(modrm)];
-  l = 2;
-  switch (D_HO(modrm)) {	// some decoding of range ptr (else CS)
-    case 0:
-	switch (D_LO(modrm)) {
-	    case 4: l++; sib=csp[2]; if (D_LO(sib)==5) l+=4; break;
-	    case 5: l+=4; bp = (long *)(*((long *)(csp+2))); break;
-	    default: break;
-	} break;
-    case 2: l+=3;
-    case 1: l++;
-	if (D_LO(modrm)==4) l++;
-	break;
-    case 3:	// register
-	bp = (long *)(((long *)&(scp->edi))[D_LO(modrm)]);
-	break;
+  if (d.emu>1) {
+	if (d.emu==3) e_printf("Scp2CpuD: %08lx -> %08lx\n\tIP=%08lx:%08lx\n%s\n",
+			scp->eflags, TheCPU.eflags, LONG_CS, scp->eip,
+			e_print_regs());
+	else e_printf("Scp2CpuD: %08lx -> %08lx\n",scp->eflags, TheCPU.eflags);
   }
-  if (bp)
-    e_printf("BOUND excp for %08lx in (%08lx..%08lx)\n",r,bp[0],bp[1]);
-  else
-    e_printf("BOUND excp for %08lx in default CS:(%08lx..%08lx)\n",r,
-	CS_DTR.BoundL,CS_DTR.BoundH);
-#ifdef SKIP_BOUND
-  _eip += l;
-  return 0;
-#else
-  leavedos(0x4244);
-  return 1;
-#endif
-#else
-  return 1;
-#endif
+  return mode;
 }
 
 
@@ -712,7 +704,8 @@ void init_emu_cpu (void)
   TheCPU.gs_cache.BoundL = 0;
   TheCPU.gs_cache.BoundH = 0x10ffff;
 
-  Reg2Cpu(&vm86s,ADDR16|DATA16|STACK16);
+  Reg2Cpu(&vm86s,ADDR16|DATA16);
+  TheCPU.StackMask = 0x0000ffff;
   init_emu_npu();
 
   switch (vm86s.cpu_type) {
@@ -745,7 +738,7 @@ static void e_gen_sigalrm(int sig, struct sigcontext_struct context)
 	 * the passed context is that of dosemu, NOT that of the
 	 * emulated CPU! */
 
-	e_signal_pending = 1;
+	e_signal_pending |= 1;
 	e_sigpa_count += sigEMUdelta;
 	if (!in_vm86 && !in_dpmi) {
 	    TheCPU.EMUtime = GETTSC();
@@ -780,16 +773,13 @@ void enter_cpu_emu(void)
 	config.cpuemu=3;	/* for saving CPU flags */
 	emu_dpmi_retcode = -1;
 	GDT = NULL; IDT = NULL;
-	  /* allocate the LDT used by dpmi (w/o GDT) */
-	  if (LDT==NULL) {
+	/* allocate the LDT used by dpmi (w/o GDT) */
+	if (LDT==NULL) {
 		alloc_LDT = LDT = (Descriptor *)calloc(LGDT_ENTRIES, sizeof(Descriptor));
 		e_printf("LDT allocated at %08lx\n",(long)LDT);
 		TheCPU.LDTR.Base = (long)LDT;
 		TheCPU.LDTR.Limit = 0xffff;
 	}
-#ifdef X_SUPPORT
-	emu_under_X = (config.X != 0);
-#endif
 #ifdef SKIP_EMU_VBIOS
 	if ((ISEG(0x10)==INT10_WATCHER_SEG)&&(IOFF(0x10)==INT10_WATCHER_OFF))
 		IOFF(0x10)=CPUEMU_WATCHER_OFF;
@@ -797,6 +787,8 @@ void enter_cpu_emu(void)
 	e_printf("EMU86: turning emuretrace ON\n");
 	last_emuretrace = config.emuretrace;
 	config.emuretrace = 2;
+	JumpOpt = 3;
+	JumpOptLim = 0x1000;
 
 	e_printf("EMU86: switching SIGALRMs\n");
 	TheCPU.EMUtime = GETTSC();
@@ -814,6 +806,8 @@ void enter_cpu_emu(void)
 	e_printf("TIME: using %d usec for updating PROF timer\n", config.realdelta);
 	setitimer(ITIMER_PROF, &itv, NULL);
 	SETSIG(SIGPROF, e_gen_sigprof);
+	NEWSETSIG(SIGFPE, e_emu_fault);
+	NEWSETSIG(SIGSEGV, e_emu_fault);
 
 	dbug_printf("======================= ENTER CPU-EMU ===============\n");
 	flush_log();
@@ -822,7 +816,10 @@ void enter_cpu_emu(void)
 
 void leave_cpu_emu(void)
 {
+	struct itimerval itv;
 	struct sigaction sa;
+	extern int MaxDepth, MaxNodes, MaxNodeSize;
+
 	if (config.cpuemu > 1) {
 		config.cpuemu=1;
 #ifdef SKIP_EMU_VBIOS
@@ -832,18 +829,35 @@ void leave_cpu_emu(void)
 		e_printf("EMU86: turning emuretrace OFF\n");
 		config.emuretrace = last_emuretrace;
 
+		NEWSETSIG(SIGFPE, dosemu_fault);
+		NEWSETSIG(SIGSEGV, dosemu_fault);
 		e_printf("EMU86: switching SIGALRMs\n");
 		NEWSETQSIG(SIGALRM, sigalrm);
+
+		itv.it_interval.tv_sec = 0;
+		itv.it_interval.tv_usec = 0;
+		itv.it_value.tv_sec = 0;
+		itv.it_value.tv_usec = 0;
+		e_printf("TIME: disabling PROF timer\n");
+		setitimer(ITIMER_PROF, &itv, NULL);
 		SETSIG(SIGPROF, SIG_IGN);
+
 		EndGen();
-		if (alloc_LDT!=NULL) free(alloc_LDT);
+		mprot_end();
+
+		if (alloc_LDT!=NULL) { free(alloc_LDT); alloc_LDT=LDT=NULL; }
 		GDT = NULL; IDT = NULL;
 		dbug_printf("======================= LEAVE CPU-EMU ===============\n");
 		dbug_printf("Total cpuemu time %16Ld us\n",TotalTime/config.CPUSpeedInMhz);
+		dbug_printf("Total exec   time %16Ld us\n",ExecTime/config.CPUSpeedInMhz);
 		dbug_printf("Total insert time %16Ld us\n",AddTime/config.CPUSpeedInMhz);
 		dbug_printf("Total search time %16Ld us\n",SearchTime/config.CPUSpeedInMhz);
-		dbug_printf("Total exec   time %16Ld us\n",ExecTime/config.CPUSpeedInMhz);
 		dbug_printf("Total clean  time %16Ld us\n",CleanupTime/config.CPUSpeedInMhz);
+		dbug_printf("Max tree nodes    %16d\n",MaxNodes);
+		dbug_printf("Max node size     %16d\n",MaxNodeSize);
+		dbug_printf("Max tree depth    %16d\n",MaxDepth);
+
+		/*re*/init_emu_cpu();
 	}
 	flush_log();
 }
@@ -973,7 +987,7 @@ int e_vm86(void)
 
   tt0 = GETTSC();
   e_sigpa_count = 0;
-  mode = ADDR16|DATA16|STACK16;
+  mode = ADDR16|DATA16; TheCPU.StackMask = 0x0000ffff;
   TheCPU.EMUtime = GETTSC();
 #ifdef SKIP_VM86_TRACE
   demusav=d.emu; if (d.emu) d.emu=1;
@@ -982,15 +996,7 @@ int e_vm86(void)
 //    e_printf("EMU86: last sig at %Ld, curr=%Ld, next=%Ld\n",lastEMUsig>>16,
 //    	TheCPU.EMUtime>>16,sigEMUtime>>16);
 
- /* This emulates VM86_ENTER only */
- /*
-  * The eflags register is also special: we cannot trust that the user
-  * has set it up safely, so this makes sure interrupt etc flags are
-  * inherited from protected mode.
-  */
-  TheCPU.veflags = REG(eflags);
-  REG(eflags) &= SAFE_MASK;
-
+ /* This emulates VM86_ENTER */
   /* ------ OUTER LOOP: exit for code >=0 and return to dosemu code */
   do {
     Reg2Cpu (&vm86s, mode);
@@ -1052,7 +1058,9 @@ int e_vm86(void)
 		struct sigcontext_struct scp;
 		Cpu2Scp (&scp, xval-1);
 		/* CALLBACK */
+		TotalTime += (GETTSC() - tt0);
 		dosemu_fault1(xval-1, &scp);
+		tt0 = GETTSC();
 		Scp2CpuR (&scp);
 		in_vm86 = 1;
 		retval = -1;	/* reenter vm86 emu */
@@ -1149,6 +1157,7 @@ int e_dpmi(struct sigcontext_struct *scp)
     E_TIME_STRETCH;
 
     if ((xval==EXCP_SIGNAL) || (xval==EXCP_PICSIGNAL) || (xval==EXCP_STISIGNAL)) {
+/**/ e_printf("DPMI retcode = %d\n",emu_dpmi_retcode);
 	if (emu_dpmi_retcode >= 0) {
 	    retval=emu_dpmi_retcode; emu_dpmi_retcode = -1;
 	}
@@ -1157,16 +1166,20 @@ int e_dpmi(struct sigcontext_struct *scp)
         retval = 0;
     }
     else {
+	TotalTime += (GETTSC() - tt0);
 	dpmi_fault(scp);
+	tt0 = GETTSC();
 	if (emu_dpmi_retcode >= 0) {
 		retval=emu_dpmi_retcode; emu_dpmi_retcode = -1;
 	}
-	else switch (scp->trapno) {
-	  case 0x6c: case 0x6d: case 0x6e: case 0x6f:
-	  case 0xe4: case 0xe5: case 0xe6: case 0xe7:
-	  case 0xec: case 0xed: case 0xee: case 0xef:
-	  case 0xfa: case 0xfb: retval = -1; break;
-	  default: retval = 0; break;
+	else {
+	  switch (scp->trapno) {
+	    case 0x6c: case 0x6d: case 0x6e: case 0x6f:
+	    case 0xe4: case 0xe5: case 0xe6: case 0xe7:
+	    case 0xec: case 0xed: case 0xee: case 0xef:
+	    case 0xfa: case 0xfb: retval = -1; break;
+	    default: retval = 0; break;
+	  }
 	}
 	scp->trapno = 0;
     }
