@@ -491,7 +491,7 @@ dos_ctrl_alt_del(void)
     keyb_server_reset();
     video_config_init();
     serial_init();
-    mouse_init();
+    dosemu_mouse_init();
     printer_init();
     disk_close();
     disk_init();
@@ -550,7 +550,7 @@ leavedos(int sig)
        longjmp(NotJEnv, sig);
       }
     leavedos_recurse_check = 1;
-    warn("leavedos(%d) called - shutting down\n", sig);
+    dbug_printf("leavedos(%d|0x%x) called - shutting down\n", sig, sig);
 #if 1 /* BUG CATCHER */
     if (in_vm86) {
       g_printf("\nkilled while in vm86(), trying to dump DOS-registers:\n");
@@ -664,7 +664,7 @@ leavedos(int sig)
     g_printf("calling serial_close\n");
     serial_close();
     g_printf("calling mouse_close\n");
-    mouse_close();
+    dosemu_mouse_close();
 
     flush_log();
 
