@@ -18,7 +18,7 @@
  *			dawes@XFree86.Org
  *
  * Changes for DOSEMU:
- *		9/97 Alberto Vignani <vignani@mail.tin.it>
+ *		9/97 Alberto Vignani <vignani@tin.it>
  *
  */
 
