@@ -284,12 +284,8 @@ void dosemu_fault(int, struct sigcontext_struct);
 #endif
 
 void show_regs(char *, int), show_ints(int, int);
-#ifdef USE_INT_QUEUE
-__inline__ int do_hard_int(int);
-#endif
 
 int cpu_trap_0f (unsigned char *, struct sigcontext_struct *);
-int e_decode_bound_excp (unsigned char *csp, struct sigcontext_struct *scp);
 
 extern unsigned int read_port_w(unsigned short port);
 extern int write_port_w(unsigned int value,unsigned short port);
