@@ -28,6 +28,7 @@
 #define MOUSE_PS2 7
 #define MOUSE_HITACHI 8
 #define MOUSE_X 9
+#define MOUSE_IMPS2 10
 
 /* types of mouse events */
 #define DELTA_CURSOR		1
@@ -146,9 +147,9 @@ EXTERN mouse_t mice[MAX_MOUSE] ;
 
 EXTERN int keyboard_mouse;
 
-extern void mouse_init(void);
+extern void dosemu_mouse_init(void);
 extern void mouse_int(void);
-extern void mouse_close(void);
+extern void dosemu_mouse_close(void);
 extern void int74(void);
 
 extern void mouse_move(void);
